@@ -4,17 +4,22 @@
 import re
 import sys
 
-PLACES_TO_CHECK = [
+PLACES_WITH_RUST_VERSION = [
     ['meson.build',                      r"msrv = '(.*)'"],
     ['Cargo.toml',                       r'rust-version\s*=\s*"(.*)"'],
     ['ci/container_builds.yml',          r'RUST_MINIMUM:\s*"(.*)"'],
     ['devel-docs/_build_dependencies.rst', r'`rust .*`_ (.*) or later'],
 ]
 
-def main():
+PLACES_WITH_CARGO_CBUILD_VERSION = [
+    ['meson.build',           r"cargo_cbuild_version = '(.*)'"],
+    ['librsvg-c/Cargo.toml',  r'min_version = "(.*)"'],
+]
+
+def check_versions(name, places):
     versions = []
 
-    for filename, regex in PLACES_TO_CHECK:
+    for filename, regex in places:
         r = re.compile(regex)
 
         with open(filename) as f:
@@ -38,14 +43,18 @@ def main():
             all_the_same = False
 
     if not all_the_same:
-        print('Version numbers do not match in these lines, please fix them!\n', file=sys.stderr)
+        print(f'{name}: Version numbers do not match in these lines, please fix them!\n', file=sys.stderr)
 
         for filename, line_number, version, line in versions:
-            print(f'{filename}:{line_number}: {line}', file=sys.stderr)
+            print(f'  {filename}:{line_number}: {line}', file=sys.stderr)
 
         sys.exit(1)
 
-    print('Versions number match.  All good!', file=sys.stderr)
+    print(f'{name}: Versions number match.  All good!', file=sys.stderr)
+
+def main():
+    check_versions('rustc', PLACES_WITH_RUST_VERSION)
+    check_versions('cargo-cbuild', PLACES_WITH_CARGO_CBUILD_VERSION)
 
 if __name__ == "__main__":
     main()
