@@ -18,15 +18,7 @@ if __name__ == '__main__':
 
     cache_file: Path = args.gdk_pixbuf_cache_file
 
-    # Install the files relative to the destdir if it's set
-    destdir = os.environ.get("DESTDIR")
-    if destdir is not None:
-        destdir: Path = Path(destdir)
-        # Make sure it's a valid Path object
-        assert destdir is not None
-        cache_file = destdir / cache_file.relative_to(cache_file.anchor)
-
-    if args.show_cross_message:
+    if args.show_cross_message or os.environ.get("DESTDIR"):
         print('*** Note: Please run gdk-pixbuf-queryloaders manually ' +
               'against the newly-built gdkpixbuf-svg loader', file=sys.stderr)
     else:
