@@ -1,4 +1,4 @@
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, ns};
 
 use crate::aspect_ratio::AspectRatio;
 use crate::document::{AcquiredNodes, Document, NodeId, Resource};
@@ -13,13 +13,15 @@ use crate::rect::Rect;
 use crate::rsvg_log;
 use crate::session::Session;
 use crate::surface_utils::shared_surface::{Interpolation, SharedImageSurface, SurfaceType};
+use crate::transform::ValidTransform;
 use crate::viewbox::ViewBox;
 use crate::xml::Attributes;
 
 use super::bounds::{Bounds, BoundsBuilder};
 use super::context::{FilterContext, FilterOutput};
 use super::{
-    FilterEffect, FilterError, FilterResolveError, Primitive, PrimitiveParams, ResolvedPrimitive,
+    FilterEffect, FilterError, FilterResolveError, InputRequirements, Primitive, PrimitiveParams,
+    ResolvedPrimitive,
 };
 
 /// The `feImage` filter primitive.
@@ -90,7 +92,7 @@ impl Image {
 
             Source::Node(node, ref name) => {
                 if let Ok(acquired) = acquired_nodes.acquire_ref(node) {
-                    rsvg_log!(draw_ctx.session(), "(feImage \"{}\"", name);
+                    rsvg_log!(ctx.session(), "(feImage \"{}\"", name);
                     let res = self.render_node(
                         ctx,
                         acquired_nodes,
@@ -98,7 +100,7 @@ impl Image {
                         bounds.clipped,
                         acquired.get(),
                     );
-                    rsvg_log!(draw_ctx.session(), ")");
+                    rsvg_log!(ctx.session(), ")");
                     res?
                 } else {
                     return Err(FilterError::InvalidInput);
@@ -116,6 +118,10 @@ impl Image {
         })
     }
 
+    pub fn get_input_requirements(&self) -> InputRequirements {
+        InputRequirements::default()
+    }
+
     /// Renders the filter if the source is an existing node.
     fn render_node(
         &self,
@@ -136,11 +142,13 @@ impl Image {
 
         let interpolation = Interpolation::from(self.feimage_values.image_rendering());
 
+        let paffine = ValidTransform::try_from(ctx.paffine())?;
+
         let image = draw_ctx.draw_node_to_surface(
             referenced_node,
             acquired_nodes,
             &cascaded,
-            ctx.paffine(),
+            paffine,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
         )?;
@@ -172,7 +180,7 @@ impl Image {
 
             Err(e) => {
                 rsvg_log!(
-                    draw_ctx.session(),
+                    ctx.session(),
                     "could not load image \"{}\" for feImage: {}",
                     url,
                     e
@@ -264,12 +272,7 @@ impl Image {
 
             let options = draw_ctx.rendering_options(SvgNesting::ReferencedFromImageElement);
 
-            document.render_document(
-                draw_ctx.session(),
-                &cr,
-                &cairo::Rectangle::from(surface_dest_rect),
-                &options,
-            )?;
+            document.render_document(&cr, &cairo::Rectangle::from(surface_dest_rect), &options)?;
         }
 
         // Now paste that image as a normal raster image
