@@ -4,7 +4,7 @@ use std::f64::consts::*;
 use std::ops::Deref;
 
 use cssparser::Parser;
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, ns};
 
 use crate::angle::Angle;
 use crate::aspect_ratio::*;
@@ -12,7 +12,7 @@ use crate::bbox::BoundingBox;
 use crate::borrow_element_as;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::{DrawingCtx, Viewport};
-use crate::element::{set_attribute, ElementTrait};
+use crate::element::{set_attribute, DrawResult, ElementTrait};
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::layout::{self, Shape, StackingContext};
@@ -122,7 +122,7 @@ impl Marker {
         clipping: bool,
         marker_type: MarkerType,
         marker: &layout::Marker,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         let mut cascaded = CascadedValues::new_from_node(node);
         cascaded.context_fill = Some(marker.context_fill.clone());
         cascaded.context_stroke = Some(marker.context_stroke.clone());
@@ -137,7 +137,7 @@ impl Marker {
         if marker_width.approx_eq_cairo(0.0) || marker_height.approx_eq_cairo(0.0) {
             // markerWidth or markerHeight set to 0 disables rendering of the element
             // https://www.w3.org/TR/SVG/painting.html#MarkerWidthAttribute
-            return Ok(draw_ctx.empty_bbox());
+            return Ok(viewport.empty_bbox());
         }
 
         let rotation = match self.orient {
@@ -160,7 +160,7 @@ impl Marker {
 
         let content_viewport = if let Some(vbox) = self.vbox {
             if vbox.is_empty() {
-                return Ok(draw_ctx.empty_bbox());
+                return Ok(viewport.empty_bbox());
             }
 
             let r = self
@@ -185,9 +185,10 @@ impl Marker {
         // FIXME: This is the only place in the code where we pass a Some(rect) to
         // StackingContext::new() for its clip_rect argument.  The effect is to clip to
         // the viewport that the current marker should establish, but this code for
-        // drawing markers does not yet use the viewports machinery and instead does
-        // things by hand.  We should encode the information about the overflow property
-        // in the viewport, so it knows whether to clip or not.
+        // drawing markers does not yet use the layout_viewport argument in the call to
+        // with_discrete_layer() below and instead does things by hand.  We should do all
+        // this with the layout_viewport instead of clip_rect, and then we can remove the
+        // clip_rect field in StackingContext.
         let clip_rect = if values.is_overflow() {
             None
         } else if let Some(vbox) = self.vbox {
@@ -609,7 +610,7 @@ fn emit_marker_by_node(
     line_width: f64,
     clipping: bool,
     marker_type: MarkerType,
-) -> Result<BoundingBox, InternalRenderingError> {
+) -> DrawResult {
     match acquired_nodes.acquire_ref(marker.node_ref.as_ref().unwrap()) {
         Ok(acquired) => {
             let node = acquired.get();
@@ -633,7 +634,7 @@ fn emit_marker_by_node(
 
         Err(e) => {
             rsvg_log!(draw_ctx.session(), "could not acquire marker: {}", e);
-            Ok(draw_ctx.empty_bbox())
+            Ok(viewport.empty_bbox())
         }
     }
 }
@@ -650,9 +651,9 @@ fn emit_marker<E>(
     marker_type: MarkerType,
     orient: Angle,
     emit_fn: &mut E,
-) -> Result<BoundingBox, InternalRenderingError>
+) -> DrawResult
 where
-    E: FnMut(MarkerType, f64, f64, Angle) -> Result<BoundingBox, InternalRenderingError>,
+    E: FnMut(MarkerType, f64, f64, Angle) -> DrawResult,
 {
     let (x, y) = match *segment {
         Segment::Degenerate { x, y } => (x, y),
@@ -672,31 +673,21 @@ pub fn render_markers_for_shape(
     draw_ctx: &mut DrawingCtx,
     acquired_nodes: &mut AcquiredNodes<'_>,
     clipping: bool,
-) -> Result<BoundingBox, InternalRenderingError> {
-    let path = match &shape.path {
-        layout::Path::Validated {
-            path,
-            extents: Some(_),
-            ..
-        } => path,
-        layout::Path::Validated { extents: None, .. } => return Ok(draw_ctx.empty_bbox()),
-        layout::Path::Invalid(_) => return Ok(draw_ctx.empty_bbox()),
-    };
-
+) -> DrawResult {
     if shape.stroke.width.approx_eq_cairo(0.0) {
-        return Ok(draw_ctx.empty_bbox());
+        return Ok(viewport.empty_bbox());
     }
 
     if shape.marker_start.node_ref.is_none()
         && shape.marker_mid.node_ref.is_none()
         && shape.marker_end.node_ref.is_none()
     {
-        return Ok(draw_ctx.empty_bbox());
+        return Ok(viewport.empty_bbox());
     }
 
     emit_markers_for_path(
-        path,
-        draw_ctx.empty_bbox(),
+        &shape.path.path,
+        viewport.empty_bbox(),
         &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: Angle| {
             let marker = match marker_type {
                 MarkerType::Start => &shape.marker_start,
@@ -718,7 +709,7 @@ pub fn render_markers_for_shape(
                     marker_type,
                 )
             } else {
-                Ok(draw_ctx.empty_bbox())
+                Ok(viewport.empty_bbox())
             }
         },
     )
@@ -726,11 +717,11 @@ pub fn render_markers_for_shape(
 
 fn emit_markers_for_path<E>(
     path: &Path,
-    empty_bbox: BoundingBox,
+    empty_bbox: Box<BoundingBox>,
     emit_fn: &mut E,
-) -> Result<BoundingBox, InternalRenderingError>
+) -> DrawResult
 where
-    E: FnMut(MarkerType, f64, f64, Angle) -> Result<BoundingBox, InternalRenderingError>,
+    E: FnMut(MarkerType, f64, f64, Angle) -> DrawResult,
 {
     enum SubpathState {
         NoSubpath,
@@ -1182,14 +1173,10 @@ mod marker_tests {
 
         assert!(emit_markers_for_path(
             &builder.into_path(),
-            BoundingBox::new(),
-            &mut |marker_type: MarkerType,
-                  x: f64,
-                  y: f64,
-                  computed_angle: Angle|
-             -> Result<BoundingBox, InternalRenderingError> {
+            Box::new(BoundingBox::new()),
+            &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: Angle| -> DrawResult {
                 v.push((marker_type, x, y, computed_angle));
-                Ok(BoundingBox::new())
+                Ok(Box::new(BoundingBox::new()))
             }
         )
         .is_ok());
@@ -1218,14 +1205,10 @@ mod marker_tests {
 
         assert!(emit_markers_for_path(
             &builder.into_path(),
-            BoundingBox::new(),
-            &mut |marker_type: MarkerType,
-                  x: f64,
-                  y: f64,
-                  computed_angle: Angle|
-             -> Result<BoundingBox, InternalRenderingError> {
+            Box::new(BoundingBox::new()),
+            &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: Angle| -> DrawResult {
                 v.push((marker_type, x, y, computed_angle));
-                Ok(BoundingBox::new())
+                Ok(Box::new(BoundingBox::new()))
             }
         )
         .is_ok());
