@@ -1,17 +1,17 @@
 //! Structural elements in SVG: the `g`, `switch`, `svg`, `use`, `symbol`, `clip_path`, `mask`, `link` elements.
 
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, ns};
 
 use crate::aspect_ratio::*;
 use crate::bbox::BoundingBox;
 use crate::coord_units;
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::{ClipMode, DrawingCtx, SvgNesting, Viewport};
-use crate::element::{set_attribute, ElementData, ElementTrait};
+use crate::drawing_ctx::{DrawingCtx, SvgNesting, Viewport};
+use crate::element::{set_attribute, DrawResult, ElementData, ElementTrait};
 use crate::error::*;
 use crate::href::{is_href, set_href};
-use crate::layout::StackingContext;
+use crate::layout::{self, Layer, LayerKind, LayoutViewport, StackingContext};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
@@ -33,26 +33,117 @@ impl ElementTrait for Group {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(
+        let stacking_ctx = Box::new(StackingContext::new(
             draw_ctx.session(),
             acquired_nodes,
             &elt,
             values.transform(),
             None,
             values,
+        ));
+
+        draw_ctx.with_discrete_layer(
+            &stacking_ctx,
+            acquired_nodes,
+            viewport,
+            None,
+            clipping,
+            &mut |an, dc, new_viewport| {
+                node.draw_children(an, cascaded, new_viewport, dc, clipping)
+            },
+        )
+    }
+
+    fn layout(
+        &self,
+        node: &Node,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
+        draw_ctx: &mut DrawingCtx,
+        clipping: bool,
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
+        let mut child_layers = Vec::new();
+
+        for child in node.children().filter(|c| c.is_element()) {
+            let elt = child.borrow_element();
+
+            let layer = elt.layout(
+                &child,
+                acquired_nodes,
+                &CascadedValues::clone_with_node(cascaded, &child),
+                viewport,
+                draw_ctx,
+                clipping,
+            )?;
+
+            if let Some(layer) = layer {
+                child_layers.push(layer);
+            }
+        }
+
+        self.layout_with_children(
+            draw_ctx.session(),
+            node,
+            acquired_nodes,
+            cascaded,
+            child_layers,
+        )
+    }
+}
+
+fn extents_of_transformed_children(layers: &[Layer]) -> Option<Rect> {
+    let mut result_bbox = BoundingBox::new();
+
+    for layer in layers {
+        if let Some(extents) = layer.kind.local_extents() {
+            let bbox = BoundingBox::new()
+                .with_transform(layer.stacking_ctx.transform)
+                .with_rect(extents);
+            result_bbox.insert(&bbox);
+        }
+    }
+
+    result_bbox.rect
+}
+
+impl Group {
+    fn layout_with_children(
+        &self,
+        session: &Session,
+        node: &Node,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        cascaded: &CascadedValues<'_>,
+        child_layers: Vec<Layer>,
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
+        let values = cascaded.get();
+
+        let extents = extents_of_transformed_children(&child_layers);
+
+        let group = Box::new(layout::Group {
+            children: child_layers,
+            establish_viewport: None,
+            extents,
+        });
+
+        let elt = node.borrow_element();
+        let stacking_ctx = StackingContext::new(
+            session,
+            acquired_nodes,
+            &elt,
+            values.transform(),
+            None,
+            values,
         );
 
-        draw_ctx.with_discrete_layer(
-            &stacking_ctx,
-            acquired_nodes,
-            viewport,
-            clipping,
-            &mut |an, dc| node.draw_children(an, cascaded, viewport, dc, clipping),
-        )
+        Ok(Some(Layer {
+            kind: LayerKind::Group(group),
+            stacking_ctx,
+        }))
     }
 }
 
@@ -78,25 +169,26 @@ impl ElementTrait for Switch {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(
+        let stacking_ctx = Box::new(StackingContext::new(
             draw_ctx.session(),
             acquired_nodes,
             &elt,
             values.transform(),
             None,
             values,
-        );
+        ));
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
             viewport,
+            None,
             clipping,
-            &mut |an, dc| {
+            &mut |an, dc, new_viewport| {
                 if let Some(child) = node.children().filter(|c| c.is_element()).find(|c| {
                     let elt = c.borrow_element();
                     elt.get_cond(dc.user_language())
@@ -104,12 +196,12 @@ impl ElementTrait for Switch {
                     child.draw(
                         an,
                         &CascadedValues::clone_with_node(cascaded, &child),
-                        viewport,
+                        new_viewport,
                         dc,
                         clipping,
                     )
                 } else {
-                    Ok(dc.empty_bbox())
+                    Ok(new_viewport.empty_bbox())
                 }
             },
         )
@@ -226,20 +318,13 @@ impl Svg {
         cascaded: &CascadedValues<'_>,
         current_viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
-    ) -> Option<Viewport> {
+    ) -> LayoutViewport {
         let values = cascaded.get();
 
         let params = NormalizeParams::new(values, current_viewport);
 
         let has_parent = node.parent().is_some();
 
-        // FIXME: do we need to look at preserveAspectRatio.slice, like in DrawingCtx::draw_image()?
-        let clip_mode = if !values.is_overflow() && has_parent {
-            ClipMode::ClipToViewport
-        } else {
-            ClipMode::NoClip
-        };
-
         // From https://www.w3.org/TR/SVG2/embedded.html#ImageElement:
         //
         // For `image` elements embedding an SVG image, the `preserveAspectRatio`
@@ -261,7 +346,7 @@ impl Svg {
 
         let is_measuring_toplevel_svg = !has_parent && draw_ctx.is_measuring();
 
-        let (viewport, vbox) = if is_measuring_toplevel_svg {
+        let (geometry, vbox) = if is_measuring_toplevel_svg {
             // We are obtaining the toplevel SVG's geometry.  This means, don't care about the
             // DrawingCtx's viewport, just use the SVG's intrinsic dimensions and see how far
             // it wants to extend.
@@ -285,13 +370,12 @@ impl Svg {
             )
         };
 
-        draw_ctx.push_new_viewport(
-            current_viewport,
+        LayoutViewport {
+            geometry,
             vbox,
-            viewport,
             preserve_aspect_ratio,
-            clip_mode,
-        )
+            overflow: values.overflow(),
+        }
     }
 }
 
@@ -318,30 +402,29 @@ impl ElementTrait for Svg {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(
+        let stacking_ctx = Box::new(StackingContext::new(
             draw_ctx.session(),
             acquired_nodes,
             &elt,
             values.transform(),
             None,
             values,
-        );
+        ));
+
+        let layout_viewport = self.make_svg_viewport(node, cascaded, viewport, draw_ctx);
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
-            viewport, // FIXME: should this be the svg_viewport from below?
+            viewport,
+            Some(layout_viewport),
             clipping,
-            &mut |an, dc| {
-                if let Some(svg_viewport) = self.make_svg_viewport(node, cascaded, viewport, dc) {
-                    node.draw_children(an, cascaded, &svg_viewport, dc, clipping)
-                } else {
-                    Ok(dc.empty_bbox())
-                }
+            &mut |an, dc, new_viewport| {
+                node.draw_children(an, cascaded, new_viewport, dc, clipping)
             },
         )
     }
@@ -413,7 +496,7 @@ impl ElementTrait for Use {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         if let Some(link) = self.link.as_ref() {
             let values = cascaded.get();
             let params = NormalizeParams::new(values, viewport);
@@ -449,7 +532,7 @@ impl ElementTrait for Use {
                 stroke_paint,
             )
         } else {
-            Ok(draw_ctx.empty_bbox())
+            Ok(viewport.empty_bbox())
         }
     }
 }
@@ -607,12 +690,12 @@ impl ElementTrait for Link {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         // If this element is inside of <text>, do not draw it.
         // The <text> takes care of it.
         for an in node.ancestors() {
             if matches!(&*an.borrow_element_data(), ElementData::Text(_)) {
-                return Ok(draw_ctx.empty_bbox());
+                return Ok(viewport.empty_bbox());
             }
         }
 
@@ -629,21 +712,97 @@ impl ElementTrait for Link {
             self.link.clone()
         };
 
-        let stacking_ctx = StackingContext::new_with_link(
+        let stacking_ctx = Box::new(StackingContext::new_with_link(
             draw_ctx.session(),
             acquired_nodes,
             &elt,
             values.transform(),
             values,
             link_target,
-        );
+        ));
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
             viewport,
+            None,
             clipping,
-            &mut |an, dc| node.draw_children(an, &cascaded, viewport, dc, clipping),
+            &mut |an, dc, new_viewport| {
+                node.draw_children(an, &cascaded, new_viewport, dc, clipping)
+            },
         )
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    use crate::accept_language::{LanguageTags, UserLanguage};
+    use crate::document::Document;
+    use crate::dpi::Dpi;
+    use crate::drawing_ctx::{RenderingConfiguration, SvgNesting};
+
+    #[test]
+    fn computes_group_extents() {
+        let document = Document::load_from_bytes(
+            br#"<?xml version="1.0" encoding="UTF-8"?>
+<svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
+  <g id="a">
+    <g transform="translate(10, 10) scale(2, 3)">
+      <rect x="0" y="0" width="5" height="10"/>
+    </g>
+    <rect x="0" y="0" width="5" height="10" transform="scale(2) translate(-10, -20)"/>
+  </g>
+</svg>
+"#,
+        );
+
+        let a = document.lookup_internal_node("a").unwrap();
+
+        let elt = a.borrow_element();
+
+        let mut acquired_nodes = AcquiredNodes::new(&document, None);
+        let cascaded = CascadedValues::new_from_node(&a);
+
+        let dpi = Dpi::new(96.0, 96.0);
+
+        let viewport = Viewport::new(dpi.clone(), 100.0, 100.0);
+
+        let surface = cairo::RecordingSurface::create(cairo::Content::ColorAlpha, None).unwrap();
+        let cr = cairo::Context::new(&surface).unwrap();
+
+        let config = RenderingConfiguration {
+            dpi,
+            cancellable: None,
+            user_language: UserLanguage::LanguageTags(LanguageTags::empty()),
+            svg_nesting: SvgNesting::Standalone,
+            measuring: false,
+            testing: true,
+        };
+
+        let mut draw_ctx = DrawingCtx::new(Session::default(), &cr, &viewport, config, Vec::new());
+
+        let layout = elt.layout(
+            &a,
+            &mut acquired_nodes,
+            &cascaded,
+            &viewport,
+            &mut draw_ctx,
+            false,
+        );
+
+        match layout {
+            Ok(Some(Layer {
+                kind: LayerKind::Group(ref group),
+                ..
+            })) => {
+                assert_eq!(group.extents, Some(Rect::new(-20.0, -40.0, 20.0, 40.0)));
+            }
+
+            Err(_) => panic!("layout should not produce an InternalRenderingError"),
+
+            _ => panic!("layout object is not a LayerKind::Group"),
+        }
+    }
+}
